@@ -84,7 +84,8 @@ except ImportError:
     np = None
 
 logger = logging.getLogger(__name__)
-
+namespace_decoder = MetaStringDecoder(".", "_")
+typename_decoder = MetaStringDecoder("$", "_")
 
 if ENABLE_FORY_CYTHON_SERIALIZATION:
     from pyfory._serialization import TypeInfo
@@ -122,6 +123,16 @@ else:
                 f"serializer={self.serializer})"
             )
 
+        def decode_namespace(self) -> str:
+            if self.namespace_bytes is None:
+                return ""
+            return self.namespace_bytes.decode(namespace_decoder)
+
+        def decode_typename(self) -> str:
+            if self.typename_bytes is None:
+                return ""
+            return self.typename_bytes.decode(typename_decoder)
+
 
 class TypeResolver:
     __slots__ = (
