@@ -466,6 +466,35 @@ def test_serialize_simple_struct(data_file_path):
     struct_round_back(data_file_path, fory, obj)
 
 
+class SomeClass:
+    f1: "SomeClass"
+    f2: Dict[str, str]
+    f3: Dict[str, str]
+
+
+def test_custom_class_roundtrip():
+    fory = pyfory.Fory(ref_tracking=True)
+    fory.register_type(SomeClass, typename="example.SomeClass")
+    obj1 = SomeClass()
+    obj1.f2 = {"k1": "v1", "k2": "v2"}
+    obj1.f1, obj1.f3 = obj1, obj1.f2
+    data1 = fory.serialize(obj1)
+    obj2 = fory.deserialize(data1)
+    data2 = fory.serialize(obj2)
+    assert data1 == data2
+    # bytes can be data serialized by other languages.
+    # due to the self-referential nature of this object,
+    # direct `==` comparison will fail.
+    # 1. Serialize `obj1` to `data1`
+    # 2. Deserialize `data1` to `obj2`
+    # 3. Serialize `obj2` to `data2`
+    # If `data1 == data2`, the round-trip preserves value equivalence.
+    # print(data1)
+    # print(data2)
+    # print(obj1)
+    # print(obj2)
+
+
 class EnumTestClass(enum.Enum):
     FOO = 0
     BAR = 1
